//+------------------------------------------------------------------+
//|                                               DMI ADX histogram oscillator.mq4 |
//|                               Copyright  2012, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//|                                            mario.jemic@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  Black
#property indicator_color2  Blue
#property indicator_color3  Red
#property indicator_width1  1
#property indicator_width2  15
#property indicator_width3  15
#property indicator_minimum -0.0001
#property indicator_maximum 0.0001

//
//
//
//
//

extern string TimeFrame         = "Current time frame"; 
extern int    CalculationPeriod = 2;
extern int    PriceType         = 1;
extern bool   alertsOn          = true;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsPushNotif   = true;
extern bool   alertsSound       = true;
extern bool   alertsEmail       = true;

//
//
//
//
//

double ADX[];
double UP[];
double DOWN[];
double trend[];
int    timeFrame;

//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,ADX);
   SetIndexBuffer(1,UP);   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,DOWN); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,trend);
   timeFrame = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" DMI ADX Filter");
return(0);
}
int deinit() { return(0); }

//
//
//
//
//

int start()
 {
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMax(Bars-counted_bars,3*timeFrame/Period());
            
   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--) 
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
	   double DMI    = iADX(NULL,timeFrame,CalculationPeriod,PriceType,MODE_PLUSDI,y)-iADX(NULL,timeFrame,CalculationPeriod,PriceType,MODE_MINUSDI,y);
             ADX[i] = iADX(NULL,timeFrame,CalculationPeriod,PriceType,MODE_MAIN,y);
             UP[i]  =  EMPTY_VALUE;
             DOWN[i]=  EMPTY_VALUE;
	          trend[i] = trend[i+1];
	          if (DMI>0) trend[i] = 1;  
	          if (DMI<0) trend[i] =-1;   
			    if (trend[i] == 1) UP[i]   = DMI; 
			    if (trend[i] ==-1) DOWN[i] = DMI;  
   } 
 manageAlerts();  
 return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
    if (alertsOn)
    {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar)); 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"UP UP UP UP UP 11");
         if (trend[whichBar] == -1) doAlert(whichBar,"DOWN DOWN DOWN 11");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," DMI ADX signal ",doWhat);
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(StringConcatenate(Symbol()," DMI ADX"),message);
          if (alertsPushNotif) SendNotification(StringConcatenate(Symbol()," DMI ADX "+message));
          if (alertsSound)     PlaySound("alert2.wav");
   }
}